class GameLayer < Joybox::Core::Layer

	def self.scene
		# Redefine Joybox::Core::Layer scene method, for create colored background
		define_singleton_method(:scene) do
			scene = CCScene.new
			bg_layer = CCLayerColor.layerWithColor(
				CcColor4B.new(255,255, 255, 255),
				width: Screen.width,
				height: Screen.height
			)
			scene << bg_layer
			scene << self.new

		end
	end

	scene

	attr_accessor :player
	attr_accessor :monsters
	attr_accessor :projectiles

	def on_enter
		@monsters = []
		@projectiles = []

		@player = Sprite.new file_name:"sprites/player.png"
		@player.position = [@player.contentSize.width/2, Screen.half_height]
		self << @player

		# Schedule monsters spawning
		schedule "add_monster", interval:1

		#handle touches
		on_touches_ended do |touches, event|
			touch = touches.any_object
			touch_location = touch.location

			create_projectile(touch_location)
		end

		schedule_update do |dt|
			detect_collisions
		end

	end

	def detect_collisions
		@projectiles.each do |projectile|
			delete_projectile = false
			@monsters.each do |monster|
				if CGRectIntersectsRect(projectile.boundingBox, monster.boundingBox)
					@monsters.delete monster
					monster.removeFromParent
					delete_projectile = true
				end
			end
			if delete_projectile
				@projectiles.delete projectile
				projectile.removeFromParent
			end
		end
	end

	def create_projectile touch_location
		projectile = Sprite.new file_name:"sprites/projectile.png", position: [20, Screen.half_height]
		projectile.tag = 2		

		offset = touch_location - projectile.position
		return if offset.x <= 0
		
		self << projectile
		@projectiles << projectile

		real_x = Screen.width + projectile.contentSize.width/2
		ratio = offset.y.to_f / offset.x.to_f
		real_y = (real_x * ratio) + projectile.position.y
		real_dest = CGPoint.new real_x, real_y

		# Determinate range
		off_real_x = real_x - projectile.position.x
		off_real_y = real_y - projectile.position.y
		length = Math.sqrt( off_real_x**2 + off_real_y**2  )
		velocity = 480.0/1.0
		real_move_duration = length / velocity

		move_action = Move.to duration: real_move_duration, position: real_dest
		move_action_done = Callback.with do |node| 
			@projectiles.delete node
			node.removeFromParent
		end

		projectile.run_action Sequence.with actions:[move_action, move_action_done]
	end

	def add_monster
		monster = Sprite.new file_name:"sprites/monster.png"
		monster.tag = 1
		min_y = monster.contentSize.height / 2
		max_y = Screen.height - monster.contentSize.height/2
		range_y = max_y - min_y
		actual_y = rand * range_y + min_y

		monster.position = [Screen.width + monster.contentSize.width/2, actual_y]
		self << monster
		@monsters << monster

		# Determinate speed of monster
		min_duration = 2.0
		max_duration = 4.0
		range_duration = max_duration - min_duration
		actual_duration = rand * range_duration + min_duration

		action_move = Move.to duration:actual_duration, position:[-monster.contentSize.width/2, actual_y]
		action_move_done = Callback.with do |node|
			@monsters.delete node
			node.removeFromParent
		end

		monster.run_action Sequence.with actions:[action_move, action_move_done]

	end
end